#include <stdio.h>
#include <pthread.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "NebulaClient.h"
#include "IOTCClient.h"
#include "AVClient.h"
#include "P2PCam/AVFRAMEINFO.h"
#include "P2PCam/AVIOCTRLDEFs.h"
#include "disposable_params.h"
#include "sample_constants.h"

//########################################################
//# Nebula configurations data
//########################################################
static char gUdid[MAX_PUBLIC_UDID_LENGTH+1];
static char gPincode[MAX_PIN_CODE_LENGTH + 1] = {0};
static char gStoken[1024] = {0};

static unsigned int gBindAbort = 0;
static unsigned int gSendCommandAbort = 0;

//########################################################
//#     Get VSaaS information from app
//########################################################
void GetCloudRecordingEndpoint(char *vsaas_info) {
    // not implement in this sample
    sprintf(vsaas_info, "{\"func\":\"setCloudRecordingEndpoint\",\"args\":{\"url\":\"https://asia-vsaasapi-tutk.kalayservice.com/vsaas/api/v1/stream/stream_url/%s?stoken=%s\"}}", gUdid, gStoken);
}

//########################################################
//# Main function
//########################################################
int main(int argc, char *argv[])
{
    int ret;
    const char *profile_str = NULL;

    if (argc != 3) {
        printf("AV_VSaaS_CLient [UDID] [stoken]\n");
        return -1;
    }

    gUdid[MAX_PUBLIC_UDID_LENGTH] = 0;
    strncpy( gUdid, argv[1], MAX_PUBLIC_UDID_LENGTH );
    strncpy(gStoken, argv[2], sizeof(gStoken)-1);

    ret = loadDisposableParams(gPincode, NULL, NULL);
    if(ret<0){
        printf("loadDisposableParams()=%d\n", ret);
        return -1;
    }

    const char *license_key = getenv(ENV_LICENSE_KEY);
    if (license_key == NULL) {
        printf("Environment variable %s must be set!!\n", ENV_LICENSE_KEY);
        return -1;
    }
    ret = TUTK_SDK_Set_License_Key(license_key);
    if (ret != TUTK_ER_NoERROR) {
        printf("TUTK_SDK_Set_License_Key()=%d ,exit...!!\n", ret);
        return 0;
    }

    //Set log file path
    LogAttr log_attr;
    memset(&log_attr, 0, sizeof(log_attr));
    char log_path[] = "./client_log.txt";
    log_attr.path = log_path;

    ret = IOTC_Set_Log_Attr(log_attr);
    printf("IOTC_Set_Log_Attr ret[%d]\n", ret);


    //IOTC Initialize
    ret = IOTC_Initialize2(0);
    printf("IOTC_Initialize2 ret[%d]\n", ret);
    if(ret != IOTC_ER_NoERROR) {
        printf("IOTC_Initialize2 exit...!!\n");
        return -1;
    }

    //Nebula Initialize
    ret = Nebula_Initialize();
    printf("Nebula_Initialize ret[%d]\n", ret);
    if (ret != NEBULA_ER_NoERROR) {
        printf("IOTCAPIs_Client exit...!!\n");
        IOTC_DeInitialize();
        return -1;
    }

    NebulaClientCtx *client_ctx;

    ret = Nebula_Client_New(gUdid, &client_ctx);
    printf("Nebula_Client_New ret[%d]\n", ret);
    if (ret != NEBULA_ER_NoERROR) {
        printf("IOTCAPIs_Client exit...!!\n");
        Nebula_DeInitialize();
        IOTC_DeInitialize();
        return -1;
    }

    //Wait until device bind success
    sleep(3);
    TUTKJsonObject *json_args = NULL;
    ret = Nebula_Client_Bind(client_ctx, gPincode, &json_args, 30000, &gBindAbort);
    printf("Nebula_Client_Bind ret[%d]\n", ret);

    if (ret != NEBULA_ER_NoERROR) {
        printf("IOTCAPIs_Client exit...!!\n");
        goto Demo_Exit;
    }

    profile_str = TUTK_Json_Obj_To_String(json_args);
    printf("[Bind_Response]\n%s\n", profile_str);

    ret = Nebula_Client_Free_Bind_Response(json_args);
    if (ret != NEBULA_ER_NoERROR)
        printf("Nebula_Client_Free_Bind_Response failed!\n");

    //Send VSaaS information to Device
    char request2[2048] = {0};
    TUTKJsonObject* json_response = NULL;
    GetCloudRecordingEndpoint(request2);
    printf("get vsaas info : \n%s\n", request2);
    ret = Nebula_Client_Send_Command(client_ctx, request2, &json_response, 10000, &gSendCommandAbort);
    printf("json_response [%d]\n\n", ret);
    if (ret != NEBULA_ER_NoERROR) {
        printf("IOTCAPIs_Client exit...!!\n");
        goto Demo_Exit;
    }

    profile_str = TUTK_Json_Obj_To_String(json_response);
    printf("[setCloudRecordingEndpoint]\n%s\n", profile_str);

    ret = Nebula_Client_Free_Send_Command_Response(json_response);

    Demo_Exit:
    ret = Nebula_Client_Delete(client_ctx); 
    Nebula_DeInitialize();
    IOTC_DeInitialize();
    printf("IOTC_DeInitialize()\n");
    return 0;
}
